@@ -14,12 +14,10 @@
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
-# License along with this program; if not, write to the
-# Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
-# Boston, MA 02110-1301, USA.
+# License along with this program; if not, see <http://www.gnu.org/licenses/>.
 """UI constants and various functions and classes that help with UI drawing."""
-import decimal
 import os
+import time
 import urllib.error
 import urllib.parse
 import urllib.request
@@ -41,12 +39,10 @@ from gi.repository.GstPbutils import DiscovererSubtitleInfo
 from gi.repository.GstPbutils import DiscovererVideoInfo
 
 from pitivi.configure import get_pixmap_dir
-from pitivi.utils.loggable import doLog
+from pitivi.utils.loggable import do_log
 from pitivi.utils.loggable import ERROR
 from pitivi.utils.loggable import INFO
 from pitivi.utils.misc import path_from_uri
-from pitivi.utils.proxy import get_proxy_target
-
 
 # Dimensions in pixels
 EXPANDED_SIZE = 65
@@ -60,7 +56,9 @@ SNAPBAR_WIDTH = 5
 SNAPBAR_COLOR = (127, 153, 204)
 LAYER_HEIGHT = 130
 # The space between two layers.
-SEPARATOR_HEIGHT = PADDING
+SEPARATOR_HEIGHT = 1
+
+CLIP_BORDER_WIDTH = 1
 
 SMALL_THUMB_WIDTH = 64
 # 128 is the normal size for thumbnails, but for *icons* it looks insane.
@@ -71,6 +69,24 @@ FILE_TARGET_ENTRY = Gtk.TargetEntry.new("text/plain", 0, 0)
 URI_TARGET_ENTRY = Gtk.TargetEntry.new("text/uri-list", 0, 0)
 EFFECT_TARGET_ENTRY = Gtk.TargetEntry.new("pitivi/effect", 0, 0)
 
+TOUCH_INPUT_SOURCES = (Gdk.InputSource.TOUCHPAD,
+                       Gdk.InputSource.TRACKPOINT,
+                       Gdk.InputSource.TABLET_PAD)
+
+
+def get_month_format_string():
+    """Returns the appropriate format string for month name in time.strftime() function."""
+    # %OB produces the month name in nominative case.
+    month_format_string = "%OB"
+    if time.strftime(month_format_string) == "%OB":
+        # %B produces the month name in genitive case.
+        month_format_string = "%B"
+    return month_format_string
+
+
+# TODO: Drop this when we depend on glibc 2.27+
+MONTH_FORMAT_STRING = get_month_format_string()
+
 
 def _get_settings(schema):
     if schema not in Gio.Settings.list_schemas():
@@ -91,25 +107,118 @@ def _get_font(font_spec, default):
     face = raw_font.rsplit(" ", 1)[0]
     return cairo.ToyFontFace(face)
 
+
 NORMAL_FONT = _get_font("font-name", "Cantarell")
 DOCUMENT_FONT = _get_font("document-font-name", "Sans")
 MONOSPACE_FONT = _get_font("monospace-font-name", "Monospace")
 
-TIMELINE_CSS = """
-    .AudioBackground {
-        background-color: #496c21;
+GREETER_PERSPECTIVE_CSS = """
+    #empty_greeter_msg_title {
+        font-size: 125%;
+        font-weight: bold;
     }
 
+    #recent_projects_listbox {
+        border: 1px solid alpha(@borders, 0.6);
+    }
+
+    #recent_projects_listbox row {
+        padding: 12px 12px 12px 12px;
+        border-bottom: 1px solid alpha(@borders, 0.2);
+    }
+
+    #recent_projects_listbox row:last-child {
+        border-bottom-width: 0px;
+    }
+
+    #project_name_label {
+        font-weight: bold;
+    }
+
+    #project_uri_label,
+    #project_last_updated_label,
+    #empty_greeter_msg_subtitle {
+        opacity: 0.55;
+    }
+
+    #recent_projects_labelbox {
+        color: @insensitive_fg_color;
+        padding-bottom: 6px;
+    }
+
+    #recent_projects_labelbox > label:backdrop {
+        color: @unfocused_insensitive_color;
+    }
+
+    #recent_projects_label {
+        font-weight: bold;
+    }
+
+    #project_thumbnail_box {
+        background-color: #181818;
+    }
+"""
+
+EDITOR_PERSPECTIVE_CSS = """
+    #resize_status {
+        font-size: 200%%;
+        background-color: black;
+        opacity: 0.8;
+    }
+
+    .LayerControlEntry:not(:focus) {
+        border: 1px solid transparent;
+        background: transparent;
+    }
+
+
+    .AudioBackground,
     .VideoBackground {
-        background-color: #2d2d2d;
+        transition: background-color 200ms ease-out, border-color 200ms ease-out;
     }
 
-    .AudioBackground:selected {
-        background-color: #1b2e0e;
+    .UriClip .AudioBackground {
+        background-color: rgb(60, 97, 43);
+        border: %(clip_border_width)spx solid shade(rgb(60, 97, 43), 1.2);
     }
 
+    .UriClip .VideoBackground {
+        background-color: rgb(25, 25, 25);
+        border: %(clip_border_width)spx solid shade(rgb(25, 25, 25), 2.5);
+    }
+
+    .TitleClip .VideoBackground {
+        background-color: rgb(94, 78, 102);
+        border: %(clip_border_width)spx solid shade(rgb(25, 25, 25), 2.5);
+    }
+
+    .AudioBackground:selected,
     .VideoBackground:selected {
-        background-color: #0f0f0f;
+        border-color: rgb(132, 131, 79)
+    }
+
+    .UriClip .AudioBackground:selected {
+        background-color: shade(rgb(60, 97, 43), 0.4);
+    }
+
+    .UriClip .VideoBackground:selected {
+        background-color: shade(rgb(25, 25, 25), 0.4);
+    }
+
+    .TitleClip .VideoBackground:selected  {
+        background-color: shade(rgb(94, 78, 102), 0.4);
+    }
+
+
+    .Thumbnail {
+        transition: opacity 200ms linear;
+        opacity: 1;
+     }
+
+    .VideoPreviewer:selected,
+    .AudioPreviewer:selected,
+    .TitlePreviewer:selected {
+        opacity: 0.15;
     }
 
     .KeyframeCurve {
@@ -118,7 +227,7 @@ TIMELINE_CSS = """
 
     .Trimbar {
         background-image: url('%(trimbar_normal)s');
-        opacity:0.5;
+        opacity: 0.5;
     }
 
     .Trimbar.left {
@@ -133,6 +242,16 @@ TIMELINE_CSS = """
         background-image: url('%(trimbar_focused)s');
     }
 
+    /* Background inside the timeline */
+    .Timeline {
+        background: shade(@theme_bg_color, 0.75);
+    }
+
+    /* Background in the layers list */
+    .Timeline viewport {
+        background: @theme_bg_color;
+    }
+
     .Clip {
     }
 
@@ -149,15 +268,64 @@ TIMELINE_CSS = """
     }
 
     .SpacedSeparator {
-        background-color: rgba(0, 0, 0, 0.1);
+        background-color: rgba(0, 0, 0, 0.3);
     }
 
     .Marquee {
         background-color: rgba(224, 224, 224, 0.7);
+        color: rgba(224, 224, 224, 1);
     }
 
-""" % ({'trimbar_normal': os.path.join(get_pixmap_dir(), "trimbar-normal.png"),
-        'trimbar_focused': os.path.join(get_pixmap_dir(), "trimbar-focused.png")})
+    .MarkersBox {
+        background-color: rgba(224, 224, 224, 0);
+    }
+
+    .Marker {
+        background-image: url('%(marker_unselected)s');
+    }
+
+    .Marker:hover {
+        background-image: url('%(marker_hovered)s');
+    }
+
+    .Marker:selected {
+        background-image: url('%(marker_hovered)s');
+    }
+
+""" % ({
+    'clip_border_width': CLIP_BORDER_WIDTH,
+    'marker_hovered': os.path.join(get_pixmap_dir(), "marker-hover.png"),
+    'marker_unselected': os.path.join(get_pixmap_dir(), "marker-unselect.png"),
+    'trimbar_focused': os.path.join(get_pixmap_dir(), "trimbar-focused.png"),
+    'trimbar_normal': os.path.join(get_pixmap_dir(), "trimbar-normal.png")})
+
+
+PREFERENCES_CSS = """
+    .sidebar list {
+        background: @content_view_bg;
+    }
+
+    .sidebar row {
+        padding: 10px 40px 10px 4px;
+    }
+
+    .prefs_list row {
+        border-left: 1px solid rgb(32, 32, 32);
+        border-right: 1px solid rgb(32, 32, 32);
+        border-bottom: 1px solid rgba(25, 25, 25, 0.2);
+    }
+
+    .prefs_list .first {
+        border-top: 1px solid rgb(32, 32, 32);
+    }
+
+    /* This covers the case for the last item in a group,
+       and the last item in the list */
+    .prefs_list,
+    .prefs_list .last {
+        border-bottom: 1px solid rgb(32, 32, 32);
+    }
+"""
 
 
 def format_framerate_value(framerate):
@@ -251,12 +419,12 @@ def pack_color_32(red, green, blue, alpha=0xFFFF):
     green = green >> 8
     blue = blue >> 8
     alpha = alpha >> 8
-    return (red << 24 | green << 16 | blue << 8 | alpha)
+    return red << 24 | green << 16 | blue << 8 | alpha
 
 
 def pack_color_64(red, green, blue, alpha=0xFFFF):
     """Packs the specified 16bit color values in a 64bit RGBA value."""
-    return (red << 48 | green << 32 | blue << 16 | alpha)
+    return red << 48 | green << 32 | blue << 16 | alpha
 
 
 def unpack_color(value):
@@ -265,7 +433,7 @@ def unpack_color(value):
     Args:
         value (int): A 32bit or 64bit RGBA value.
     """
-    if not (value >> 32):
+    if not value >> 32:
         return unpack_color_32(value)
     else:
         return unpack_color_64(value)
@@ -294,9 +462,9 @@ def hex_to_rgb(value):
 
 
 def set_cairo_color(context, color):
-    if type(color) is Gdk.RGBA:
+    if isinstance(color, Gdk.RGBA):
         cairo_color = (float(color.red), float(color.green), float(color.blue))
-    elif type(color) is tuple:
+    elif isinstance(color, tuple):
         # Cairo's set_source_rgb function expects values from 0.0 to 1.0
         cairo_color = [max(0, min(1, x / 255.0)) for x in color]
     else:
@@ -311,6 +479,7 @@ def beautify_asset(asset):
     Args:
         asset (GES.Asset): The asset to display.
     """
+    from pitivi.utils.proxy import get_proxy_target
     uri = get_proxy_target(asset).props.id
     path = path_from_uri(uri)
     res = ["<b>" + GLib.markup_escape_text(path) + "</b>"]
@@ -334,7 +503,7 @@ def beautify_asset(asset):
         try:
             beautified_string = beautify_stream(stream)
         except NotImplementedError:
-            doLog(ERROR, "Beautify", "None", "Cannot beautify %s", stream)
+            do_log(ERROR, "Beautify", "None", "Cannot beautify %s", stream)
             continue
         if beautified_string:
             res.append(beautified_string)
@@ -344,7 +513,8 @@ def beautify_asset(asset):
         res.append(_("<b>Duration:</b> %s") % duration)
 
     if asset.creation_progress < 100:
-        res.append(_("<b>Proxy creation progress:</b> %d%%") % asset.creation_progress)
+        res.append(_("<b>Proxy creation progress:</b> %d%%") %
+                   asset.creation_progress)
 
     return "\n".join(res)
 
@@ -365,7 +535,8 @@ def beautify_missing_asset(asset):
 
     size = asset.get_meta("file-size")
     if size:
-        file_size = GLib.format_size_full(size, GLib.FormatSizeFlags.LONG_FORMAT)
+        file_size = GLib.format_size_full(
+            size, GLib.FormatSizeFlags.LONG_FORMAT)
         res.append(_("<b>Size</b>: %s") % file_size)
 
     return "\n".join(res)
@@ -378,7 +549,9 @@ def info_name(info):
         info (GES.Asset or DiscovererInfo): The info to display.
     """
     if isinstance(info, GES.Asset):
-        filename = urllib.parse.unquote(os.path.basename(get_proxy_target(info).get_id()))
+        from pitivi.utils.proxy import get_proxy_target
+        filename = urllib.parse.unquote(
+            os.path.basename(get_proxy_target(info).get_id()))
     elif isinstance(info, DiscovererInfo):
         filename = urllib.parse.unquote(os.path.basename(info.get_uri()))
     else:
@@ -386,8 +559,16 @@ def info_name(info):
     return GLib.markup_escape_text(filename)
 
 
+def beautify_project_path(path):
+    """Beautifies project path by shortening the home directory path (if present)."""
+    home_dir = os.path.expanduser("~")
+    if path.startswith(home_dir):
+        return path.replace(home_dir, "~")
+    return path
+
+
 def beautify_stream(stream):
-    if type(stream) is DiscovererAudioInfo:
+    if isinstance(stream, DiscovererAudioInfo):
         if stream.get_depth() == 0:
             return None
 
@@ -398,10 +579,10 @@ def beautify_stream(stream):
         return templ % (stream.get_channels(), stream.get_sample_rate(),
                         stream.get_depth())
 
-    elif type(stream) is DiscovererVideoInfo:
+    elif isinstance(stream, DiscovererVideoInfo):
         par = stream.get_par_num() / stream.get_par_denom()
-        width = stream.get_width()
-        height = stream.get_height()
+        width = stream.get_natural_width()
+        height = stream.get_natural_height()
         if not stream.is_image():
             fps = format_framerate_value(stream)
             templ = _("<b>Video:</b> %d×%d <i>pixels</i> at %s <i>fps</i>")
@@ -410,17 +591,17 @@ def beautify_stream(stream):
             templ = _("<b>Image:</b> %d×%d <i>pixels</i>")
             return templ % (par * width, height)
 
-    elif type(stream) is DiscovererSubtitleInfo:
+    elif isinstance(stream, DiscovererSubtitleInfo):
         # Ignore subtitle streams
         return None
 
-    elif type(stream) is DiscovererStreamInfo:
+    elif isinstance(stream, DiscovererStreamInfo):
         caps = stream.get_caps().to_string()
         if caps in ("application/x-subtitle", "application/x-id3", "text"):
             # Ignore all audio ID3 tags and subtitle tracks, we don't show them
             return None
 
-    raise NotImplementedError
+    raise ValueError("Unsupported stream type: %s" % stream)
 
 
 def time_to_string(value):
@@ -499,7 +680,7 @@ def beautify_time_delta(seconds):
     return ", ".join(parts)
 
 
-def beautify_ETA(length_nanos):
+def beautify_eta(length_nanos):
     """Converts the specified duration to a fuzzy estimate.
 
     Intended for progress ETAs, not to indicate a clip's duration.
@@ -522,8 +703,55 @@ def beautify_ETA(length_nanos):
     return ", ".join(parts)
 
 
+def beautify_last_updated_timestamp(last_updated_timestamp):
+    """Returns a rough estimation of how long ago the timestamp is."""
+    delta_seconds = int(time.time()) - last_updated_timestamp
+
+    if delta_seconds < 60 * 45:
+        return _("Just now")
+
+    if delta_seconds < 60 * 90:
+        return _("An hour ago")
+
+    if delta_seconds < 60 * 60 * 24:
+        return _("Today")
+
+    if delta_seconds < 60 * 60 * 24 * 2:
+        return _("Yesterday")
+
+    if delta_seconds < 60 * 60 * 24 * 7:
+        return time.strftime("%A", time.localtime(last_updated_timestamp))
+
+    if delta_seconds < 60 * 60 * 24 * 365:
+        return time.strftime(MONTH_FORMAT_STRING, time.localtime(last_updated_timestamp))
+
+    if delta_seconds < 60 * 60 * 24 * 365 * 1.5:
+        return _("About a year ago")
+
+    years = max(2, delta_seconds / (60 * 60 * 24 * 365))
+    return ngettext("About %d year ago",
+                    "About %d years ago", years) % years
+
+
 # -------------------- Gtk widget helpers ----------------------------------- #
 
+class BinWithNaturalWidth(Gtk.Bin):
+    """A bin with a maximum width."""
+
+    def __init__(self, child, width, *args, **kwargs):
+        Gtk.Bin.__init__(self, *args, **kwargs)
+        self.natural_width = width
+        self.add(child)
+
+    def do_get_request_mode(self):
+        return Gtk.SizeRequestMode.HEIGHT_FOR_WIDTH
+
+    def do_get_preferred_width(self):
+        minimum, _ = Gtk.Bin.do_get_preferred_width(self)
+        minimum = min(minimum, self.natural_width)
+        return minimum, self.natural_width
+
+
 def clear_styles(widget):
     """Makes sure the widget has no border, background or other decorations.
 
@@ -535,13 +763,34 @@ def clear_styles(widget):
         style.remove_class(css_class)
 
 
-def model(columns, data):
+def create_model(columns, data):
     ret = Gtk.ListStore(*columns)
     for datum in data:
         ret.append(datum)
     return ret
 
 
+def create_frame_rates_model(*extra_frames):
+    """Create a framerate model from the list of standard frames list and extra frames(if any).
+
+    Args:
+        extra_frames (tuple): extra frames to include in model.
+    """
+    final_list = list(standard_frames_list)
+    for frame in extra_frames:
+        if frame not in final_list:
+            final_list.append(frame)
+    final_list.sort(key=lambda x: x[0] / x[1])
+
+    items = []
+    for fps in final_list:
+        fraction = Gst.Fraction(*fps)
+        item = (format_framerate(fraction), fraction)
+        items.append(item)
+
+    return create_model((str, object), items)
+
+
 def set_combo_value(combo, value):
     def select_specific_row(model, unused_path, iter_, found):
         model_value = model.get_value(iter_, 1)
@@ -555,9 +804,9 @@ def set_combo_value(combo, value):
     combo.props.model.foreach(select_specific_row, found)
 
     if len(found) != 1:
-        doLog(INFO, None, "utils",
-              "Could not set value %s, possible values: %s",
-              (value, [v[0] for v in combo.props.model]))
+        do_log(INFO, None, "utils",
+               "Could not set value %s, possible values: %s",
+               (value, [v[1] for v in combo.props.model]))
         return False
 
     return True
@@ -595,13 +844,21 @@ def unset_children_state_recurse(widget, state):
             unset_children_state_recurse(child, state)
 
 
-def disable_scroll(widget):
-    """Makes sure the specified widget does not react to scroll events."""
-    def scroll_event_cb(widget, unused_event):
-        GObject.signal_stop_emission_by_name(widget, "scroll-event")
-        return False
+def disable_scroll_event_cb(widget, unused_event):
+    GObject.signal_stop_emission_by_name(widget, "scroll-event")
+    return False
 
-    widget.connect("scroll-event", scroll_event_cb)
+
+def disable_scroll(widget):
+    """Disables scrolling on the specified widget and its children recursively.
+
+    Makes sure the vulnerable widgets do not react to scroll events.
+    """
+    if isinstance(widget, Gtk.Container):
+        widget.foreach(disable_scroll)
+
+    if isinstance(widget, (Gtk.ComboBox, Gtk.Scale, Gtk.SpinButton)):
+        widget.connect("scroll-event", disable_scroll_event_cb)
 
 
 def fix_infobar(infobar):
@@ -613,34 +870,67 @@ def fix_infobar(infobar):
     infobar.forall(make_sure_revealer_does_nothing)
 
 
-audio_channels = model((str, int),
-    [(format_audiochannels(ch), ch) for ch in (8, 6, 4, 2, 1)])
+AUDIO_CHANNELS = create_model((str, int),
+                              [(format_audiochannels(ch), ch)
+                               for ch in (8, 6, 4, 2, 1)])
 
-frame_rates = model((str, object),
-    [(format_framerate(Gst.Fraction(*fps)), Gst.Fraction(*fps)) for fps in (
-        (12, 1),
-        (15, 1),
-        (20, 1),
-        (24000, 1001),
-        (24, 1),
-        (25, 1),
-        (30000, 1001),
-        (30, 1),
-        (50, 1),
-        (60000, 1001),
-        (60, 1),
-        (120, 1)
-    )])
+standard_frames_list = [(12, 1),
+                        (15, 1),
+                        (20, 1),
+                        (24000, 1001),
+                        (24, 1),
+                        (25, 1),
+                        (30000, 1001),
+                        (30, 1),
+                        (50, 1),
+                        (60000, 1001),
+                        (60, 1),
+                        (120, 1)
+                        ]
 
-audio_rates = model((str, int),
-    [(format_audiorate(rate), rate) for rate in (
-        8000,
-        11025,
-        12000,
-        16000,
-        22050,
-        24000,
-        44100,
-        48000,
-        96000
-    )])
+AUDIO_RATES = create_model((str, int),
+                           [(format_audiorate(rate), rate) for rate in (
+                               8000,
+                               11025,
+                               12000,
+                               16000,
+                               22050,
+                               24000,
+                               44100,
+                               48000,
+                               96000
+                           )])
+
+# This whitelist is made from personal knowledge of file extensions in the wild,
+# from gst-inspect |grep demux,
+# http://en.wikipedia.org/wiki/Comparison_of_container_formats and
+# http://en.wikipedia.org/wiki/List_of_file_formats#Video
+# ...and looking at the contents of /usr/share/mime
+SUPPORTED_FILE_FORMATS = {
+    "video": ("3gpp", "3gpp2", "dv", "mp2t", "mp2t", "mp4", "mpeg", "ogg",
+              "quicktime", "webm", "x-flv", "x-matroska", "x-mng", "x-ms-asf",
+              "x-ms-wmp", "x-ms-wmv", "x-msvideo", "x-ogm+ogg", "x-theora+ogg"),
+    "application": ("mxf",),
+    "audio": ("aac", "ac3", "basic", "flac", "mp2", "mp4", "mpeg", "ogg",
+              "opus", "webm", "x-adpcm", "x-aifc", "x-aiff", "x-aiffc",
+              "x-ape", "x-flac+ogg", "x-m4b", "x-matroska", "x-ms-asx",
+              "x-ms-wma", "x-speex", "x-speex+ogg", "x-vorbis+ogg", "x-wav"),
+    "image": ("jp2", "jpeg", "png", "svg+xml")}
+
+SUPPORTED_MIMETYPES = []
+for category, mime_types in SUPPORTED_FILE_FORMATS.items():
+    for mime in mime_types:
+        SUPPORTED_MIMETYPES.append(category + "/" + mime)
+
+
+def filter_unsupported_media_files(filter_info):
+    """Returns whether the specified item should be displayed."""
+    from pitivi.utils.proxy import ProxyManager
+
+    if filter_info.mime_type not in SUPPORTED_MIMETYPES:
+        return False
+
+    if ProxyManager.is_proxy_asset(filter_info.uri):
+        return False
+
+    return True
